
#include <ros/ros.h>
#include <visualization_msgs/Marker.h>
#include <visualization_msgs/MarkerArray.h>
#include <tf2_ros/static_transform_broadcaster.h>
#include <geometry_msgs/TransformStamped.h>
#include <geometry_msgs/Pose.h>



class ObjectInfo {
private:
    std::string objPath;
    unsigned int id;
    geometry_msgs::Pose pose;
    double scale;
    double r;
    double g;
    double b;
public:
    inline void setObjPath(std::string p) {objPath=p;}
    inline std::string getObjPath() {return objPath;}

    inline void setObjID(unsigned int i) {id=i;}
    inline unsigned int getObjID() {return id;}
    inline double getx() {return pose.position.x;}
    inline double gety() {return pose.position.y;}
    inline double getz() {return pose.position.z;}
    inline double getqx() {return pose.orientation.x;}
    inline double getqy() {return pose.orientation.y;}
    inline double getqz() {return pose.orientation.z;}
    inline double getqw() {return pose.orientation.w;}
    inline void setx(double v) {pose.position.x=v;}
    inline void sety(double v) {pose.position.y=v;}
    inline void setz(double v) {pose.position.z=v;}
    inline void setqx(double v) {pose.orientation.x=v;}
    inline void setqy(double v) {pose.orientation.y=v;}
    inline void setqz(double v) {pose.orientation.z=v;}
    inline void setqw(double v) {pose.orientation.w=v;}
    inline double getr() {return r;}
    inline double getg() {return g;}
    inline double getb() {return b;}
    inline double getscale() {return scale;}
    inline void setr(double v) {r=v;}
    inline void setg(double v) {g=v;}
    inline void setb(double v) {b=v;}
    inline void setscale(double v) {scale=v;}
};
std::vector<ObjectInfo> objects;



void SetWorld()
{
    ObjectInfo obj;

    obj.setObjPath("package://urdf_tutorial/meshes/obstacles/fixtures/Wooden_Plate.dae");
    obj.setObjID(2);
    obj.setx(0.41);
    obj.sety(0);
    obj.setz(0.015);
    obj.setqx(0);
    obj.setqy(0);
    obj.setqz(0);
    obj.setqw(1);
    obj.setr((double)rand() / (double)RAND_MAX);
    obj.setg((double)rand() / (double)RAND_MAX);
    obj.setb((double)rand() / (double)RAND_MAX);
    obj.setscale(1.0);
    objects.push_back(obj);


    obj.setObjPath("package://urdf_tutorial/meshes/obstacles/plane/UpperBody.stl");
    obj.setObjID(14);
    obj.setx(0.5);
    obj.sety(0.0); //0.4);
    obj.setz(0.04);
    obj.setqx(0);
    obj.setqy(0);
    obj.setqz(sin(M_PI /4));
    obj.setqw(cos(M_PI /4));
    obj.setr((double)rand() / (double)RAND_MAX);
    obj.setg((double)rand() / (double)RAND_MAX);
    obj.setb((double)rand() / (double)RAND_MAX);
    obj.setscale(1.0);
    objects.push_back(obj);

/*
    static tf2_ros::StaticTransformBroadcaster static_broadcaster;
    geometry_msgs::TransformStamped static_transformStamped;

    static_transformStamped.header.stamp = ros::Time::now();
    static_transformStamped.header.frame_id = "assembly_frame";
    static_transformStamped.child_frame_id = "object14";
    static_transformStamped.transform.translation.x = obj.getx();
    static_transformStamped.transform.translation.y = obj.gety();
    static_transformStamped.transform.translation.z = obj.getz();
    static_transformStamped.transform.rotation.x = obj.getqx();
    static_transformStamped.transform.rotation.y = obj.getqy();
    static_transformStamped.transform.rotation.z = obj.getqz();
    static_transformStamped.transform.rotation.w = obj.getqw();
    static_broadcaster.sendTransform(static_transformStamped);
*/
}





int main( int argc, char** argv )
{
    ros::init(argc, argv, "urdf_tutorial2");
    ros::NodeHandle n;
    ros::Rate r(1);

    ROS_INFO("**** urdf_tutorial ****");

    SetWorld();

    ros::Publisher vis_pub = n.advertise<visualization_msgs::MarkerArray>( "visualization_marker_array", 1 );

    std::vector <float> pos;
    int i;
    while (ros::ok())
    {
        i = 0;
        visualization_msgs::Marker marker;
        visualization_msgs::MarkerArray markers;


        ROS_DEBUG("LOADING OBJECTS");

        for(i=0; i<objects.size(); i++) {
            marker.header.frame_id = "assembly_frame";
            marker.header.stamp = ros::Time();
            marker.ns = "assembly_objects";
            marker.id = objects[i].getObjID();
            marker.mesh_resource = objects[i].getObjPath();

            marker.type = visualization_msgs::Marker::MESH_RESOURCE;
            marker.action = visualization_msgs::Marker::ADD;


            marker.pose.position.x = objects[i].getx();
            marker.pose.position.y = objects[i].gety();
            marker.pose.position.z = objects[i].getz();
            marker.pose.orientation.x = objects[i].getqx();
            marker.pose.orientation.y = objects[i].getqy();
            marker.pose.orientation.z = objects[i].getqz();
            marker.pose.orientation.w = objects[i].getqw();
            marker.scale.x = objects[i].getscale();
            marker.scale.y = objects[i].getscale();
            marker.scale.z = objects[i].getscale();
            marker.color.a = 1.0; // Don't forget to set the alpha!
            marker.color.r = objects[i].getr();
            marker.color.g = objects[i].getg();
            marker.color.b = objects[i].getb();


            ROS_DEBUG("OBJECT: [%d]", marker.id);
            ROS_DEBUG("X value is: [%f]", marker.pose.position.x);
            ROS_DEBUG("Y value is: [%f]", marker.pose.position.y);
            ROS_DEBUG("Z value is: [%f]", marker.pose.position.z);

            ROS_DEBUG("ORI X value is: [%f]", marker.pose.orientation.x);
            ROS_DEBUG("ORI Y value is: [%f]", marker.pose.orientation.y);
            ROS_DEBUG("ORI Z value is: [%f]", marker.pose.orientation.z);
            ROS_DEBUG("ORI W value is: [%f]", marker.pose.orientation.w);

            markers.markers.push_back(marker);
        }
        //Add a sphere centered at object1
        marker.header.frame_id = "assembly_frame";
        marker.header.stamp = ros::Time();
        marker.ns = "assembly_objects";
        marker.id = objects[1].getObjID()+1;
        marker.type = visualization_msgs::Marker::SPHERE;
        marker.action = visualization_msgs::Marker::ADD;

        marker.pose.position.x = objects[1].getx();
        marker.pose.position.y = objects[1].gety();
        marker.pose.position.z = objects[1].getz();
        marker.pose.orientation.x = objects[1].getqx();
        marker.pose.orientation.y = objects[1].getqy();
        marker.pose.orientation.z = objects[1].getqz();
        marker.pose.orientation.w = objects[1].getqw();
        marker.scale.x = 0.5;
        marker.scale.y = 0.5;
        marker.scale.z = 0.5;
        marker.color.a = 0.5; // Don't forget to set the alpha!
        marker.color.r = 1.0;
        marker.color.g = 1.0;
        marker.color.b = 1.0;
        markers.markers.push_back(marker);

        vis_pub.publish( markers );

        r.sleep();
    }
}
